Title csv  -  C86 version


; Note: csv is a symbol generated by the Minix and PC-IX C-compilers.
;	It is used to do procedure entry/exit. The .s version checks
;	the stack. In C86 no such symbol is generated by the compiler
;	so we can't use these routines.
;
;	If you want to do stack-checking, you can use the procedure
;	'$entry'. This symbol is generated by C86 when you specify
;	the '-t' flag when compiling. This routine is primairily ment
;	for stack-tracing etc. You must adapt csv then for C86 and call
;	it $entry. $entry is called after push bp and allocation for
;	local storage (sub sp,nn)

END
