/* The following names are synonyms for the variables in the input message. */
#define acc_time      m.m2_l1
#define addr	      m.m1_i3
#define buffer	      m.m1_p1
#define cd_flag	      m.m1_i2
#define child	      m.m1_i2
#define co_mode	      m.m1_i1
#define eff_grp_id    m.m1_i3
#define eff_user_id   m.m1_i3
#define erki          m.m1_p1
#define fd	      m.m1_i1
#define fd2	      m.m1_i2
#define ioflags       m.m1_i3
#define group	      m.m1_i3
#define real_grp_id   m.m1_i2
#define ls_fd	      m.m2_i1
#define mk_mode	      m.m1_i2
#define mode	      m.m3_i2
#define name	      m.m3_p1
#define name1	      m.m1_p1
#define name2	      m.m1_p2
#define	name_length   m.m3_i1
#define name1_length  m.m1_i1
#define name2_length  m.m1_i2
#define nbytes        m.m1_i2
#define offset	      m.m2_l1
#define owner	      m.m1_i2
#define parent	      m.m1_i1
#define pathname      m.m3_ca1
#define pro	      m.m1_i1
#define rd_only	      m.m1_i3
#define real_user_id  m.m1_i2
#define request       m.m1_i2
#define sig	      m.m1_i2
#define slot1	      m.m1_i1
#define tp	      m.m2_l1
#define update_time   m.m2_l2
#define utime_file    m.m2_p1
#define utime_length  m.m2_i1
#define whence	      m.m2_i2

/* The following names are synonyms for the variables in the output message. */
#define reply_type    m1.m_type
#define reply_l1      m1.m2_l1
#define reply_i1      m1.m1_i1
#define reply_i2      m1.m1_i2
#define reply_t1      m1.m4_l1
#define reply_t2      m1.m4_l2
#define reply_t3      m1.m4_l3
#define reply_t4      m1.m4_l4
